/*
  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
  dedicated to making software imaging solutions freely available.

  You may not use this file except in compliance with the License.
  obtain a copy of the License at

    http://www.imagemagick.org/script/license.php

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  MagickCore private token methods.
*/
#ifndef _MAGICKCORE_TOKEN_PRIVATE_H
#define _MAGICKCORE_TOKEN_PRIVATE_H

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

static long GetUTFOctets(const char *text,size_t *octets)
{
  long
    code;

  code=text[0];
  if ((code & 0x80) == 0)
    {
      *octets=1;
      return(code);
    }
  if ((text[1] == 0) || ((text[1] & 0xc0) != 0x80))
    {
      *octets=0;
      return(-1);
    }
  if ((code & 0xe0) != 0xe0)
    {
      *octets=2;
      code=(text[0] & 0x1f) << 6;
      code|=text[1] & 0x3f;
      return(code);
    }
  if ((text[2] == 0) || ((text[2] & 0xc0) != 0x80))
    {
      *octets=0;
      return(-1);
    }
  if ((code & 0xf0) != 0xf0)
    {
      *octets=3;
      code=(text[0] & 0x0f) << 12;
      code|=(text[1] & 0x3f) << 6;
      code|=text[2] & 0x3f;
      return(code);
    }
  if ((text[3] == 0) || ((code & 0xf8) != 0xf0) || ((text[3] & 0xc0) != 0x80))
    {
      *octets=0;
      return(-1);
    }
  *octets=4;
  code=(text[0] & 0x07) << 18;
  code|=(text[1] & 0x3f) << 12;
  code|=(text[2] & 0x3f) << 6;
  code|=text[3] & 0x3f;
  return(code);
}

static long GetUTFCode(const char *text)
{
  size_t
    octets;

  return(GetUTFOctets(text,&octets));
}

static char *GetNextUTFCode(const char *text)
{
  size_t
    octets;

  (void) GetUTFOctets(text,&octets);
  return((char *) text+octets);
}

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif
